# How custom models work

There's a lot of bits and pieces, so let's go through what happens when you drag in a custom model. 


## DragReceiverSCNView and SceneGrid.swift
This is where it starts. Rotato knows nothing about your new model until you drag it in, and this will change Rotato's perception of itself, you, the user, and what it can do. Truly a transformative event for everyone involved. 

Overall flow when loading a custom model
1. Set RotatoState's `deviceScene` to `DeviceSceneStore`'s `custom`
2. Set RotatoState's `customModel` to our `CustomModel` instance, which holds the URL to the custom scene


### SceneGrid SwiftUI view receives the custom model

1. `SceneGrid` has an `onDrop` of public file-url (and even models so you can theoretically drag a 3D model straight from Cinema 4D if they support it)
2. The `onDrop` sets `SplashController` as its delegate

### SplashController tells RotatoState about it and creates the necessary objects

1. Conforms to `DropDelegate` and reacts with `performDrop`
2. Extracts the first file from `DropInfo` in a thread, and returns true immediately if there is a file
3. Calls `self:openScene:withCustomModelUrl`
4. Creates a new document with `newDoc`
5. On that new doc, sets on RotatoState a custom scene holder through its static `DeviceSceneStore.custom`. 

_Okay, hang on, let's talk about this key scene. When `DeviceSceneStore` initializes, it makes a new scene and puts it into the `custom` property This new scene disables clay, deviceColor, labels, shadows and sets `isCustom` to true. All the other scenes are also created, but through `addScene:DeviceScene`. Another key is the `release` which is special, as this feature will expire exactly 1 year after buying Rotato, no matter the release date._

Okay, 

### RotatoState broadcasts the scene change event

6. We now create a `CustomScene:withSceneUrl:modelUrl:name`  and set RotatoState's `customScene`. The `CustomScene` is not a subclass, but a simple class that contains the scene and its name (for now). Its main purpose is being a container that can be stored in RotatoState, but also a separate file, e.g. `.rotatoScene` if needed later.

We'll now turn the control over to RotatoState's DeviceScene. 
7. When RotatoState's `deviceScene` is set, it fires `.deviceSceneWillChange`


### CanvasViewController starts preparing the scene

8. `CanvasViewController` catches that event and calls `self:changeDeviceScene`
9. At this point, we have an `SCNScene` that we want to load. CVC calls `SceneHelper: changeDeviceScene:for:SCNScene, to:DeviceScene`

### SceneHelper loads the scene from disk and inserts it into the Master Scene

10. The first thing `changeDeviceScene` does is that it checks if this `DeviceScene` is a custom scene. 
11. If it is, it takes the `customScene` from `RotatoState` and calls `CustomModelLoader:addModel:fromScene:toScene` 

### Custom Model Loader:addModel

12. Okay, the custom model loader's job is to insert our custom model SCNScene into the CustomDeviceSCene
13. First, it reshapes the customScene to make sure it has the right format for Rotato to understand it. Basically it involves adding a new top level node

